@@ -52,6 +52,7 @@
 #include "utils/ag_cache.h"
 #include "utils/agtype.h"
 #include "utils/graphid.h"
+#include "utils/name_validation.h"
 
 /*
  * Relation name doesn't have to be label name but the same name is used so
@@ -226,7 +227,7 @@ Datum create_elabel(PG_FUNCTION_ARGS)
     {
         ereport(ERROR,
                 (errcode(ERRCODE_UNDEFINED_SCHEMA),
-                        errmsg("graph \"%s\" does not exist.", graph_name_str)));
+                 errmsg("graph \"%s\" does not exist.", graph_name_str)));
     }
 
     graph_oid = get_graph_oid(graph_name_str);
@@ -259,8 +260,8 @@ Datum create_elabel(PG_FUNCTION_ARGS)
  * new table and sequence. Returns the oid from the new tuple in
  * ag_catalog.ag_label.
  */
-void create_label(char *graph_name, char *label_name, char label_type,
-                  List *parents)
+Oid create_label(char *graph_name, char *label_name, char label_type,
+                 List *parents)
 {
     graph_cache_data *cache_data;
     Oid graph_oid;
@@ -271,6 +272,13 @@ void create_label(char *graph_name, char *label_name, char label_type,
     RangeVar *seq_range_var;
     int32 label_id;
     Oid relation_id;
+    Oid label_oid;
+
+    if (!is_valid_label(label_name, label_type))
+    {
+        ereport(ERROR, (errcode(ERRCODE_UNDEFINED_SCHEMA),
+                        errmsg("label name is invalid")));
+    }
 
     cache_data = search_graph_name_cache(graph_name);
     if (!cache_data)
@@ -306,9 +314,12 @@ void create_label(char *graph_name, char *label_name, char label_type,
     // get a new "id" for the new label
     label_id = get_new_label_id(graph_oid, nsp_id);
 
-    insert_label(label_name, graph_oid, label_id, label_type, relation_id);
+    label_oid = insert_label(label_name, graph_oid, label_id, label_type,
+                             relation_id,seq_name);
 
     CommandCounterIncrement();
+
+    return label_oid;
 }
 
 // CREATE TABLE `schema_name`.`rel_name` (
@@ -673,10 +684,10 @@ static int32 get_new_label_id(Oid graph_oid, Oid nsp_id)
 
     for (cnt = LABEL_ID_MIN; cnt <= LABEL_ID_MAX; cnt++)
     {
-        int32 label_id;
+        int64 label_id;
 
         // the data type of the sequence is integer (int4)
-        label_id = (int32) nextval_internal(seq_id, true);
+        label_id = nextval_internal(seq_id, true);
         Assert(label_id_is_valid(label_id));
         if (!label_id_exists(graph_oid, label_id))
             return (int32)label_id;
