@@ -20,20 +20,23 @@
 #include "postgres.h"
 
 #include "access/htup_details.h"
-#include "access/table.h"
 #include "access/xact.h"
 #include "executor/tuptable.h"
 #include "nodes/execnodes.h"
 #include "nodes/extensible.h"
 #include "nodes/nodes.h"
 #include "nodes/plannodes.h"
+#include "parser/parse_relation.h"
+#include "rewrite/rewriteHandler.h"
 #include "utils/rel.h"
+#include "utils/tqual.h"
 
 #include "catalog/ag_label.h"
 #include "executor/cypher_executor.h"
 #include "executor/cypher_utils.h"
 #include "nodes/cypher_nodes.h"
 #include "utils/agtype.h"
+#include "utils/ag_cache.h"
 #include "utils/graphid.h"
 
 static void begin_cypher_merge(CustomScanState *node, EState *estate,
@@ -87,8 +90,7 @@ static void begin_cypher_merge(CustomScanState *node, EState *estate,
     ExecAssignExprContext(estate, &node->ss.ps);
 
     ExecInitScanTupleSlot(estate, &node->ss,
-                          ExecGetResultType(node->ss.ps.lefttree),
-                          &TTSOpsVirtual);
+                          ExecGetResultType(node->ss.ps.lefttree));
 
     /*
      * When MERGE is not the last clause in a cypher query. Setup projection
@@ -112,7 +114,7 @@ static void begin_cypher_merge(CustomScanState *node, EState *estate,
         Relation rel;
 
         /*
-         * This entity is refrences an entity that is already declared. Either
+         * This entity is references an entity that is already declared. Either
          * by a previous clause or an entity earlier in the MERGE path. In both
          * cases, this target_entry will not create data, only reference data
          * that already exists.
@@ -122,8 +124,8 @@ static void begin_cypher_merge(CustomScanState *node, EState *estate,
             continue;
         }
 
-        // Open relation and aquire a row exclusive lock.
-        rel = table_open(cypher_node->relid, RowExclusiveLock);
+        // Open relation and acquire a row exclusive lock.
+        rel = heap_open(cypher_node->relid, RowExclusiveLock);
 
         // Initialize resultRelInfo for the vertex
         cypher_node->resultRelInfo = makeNode(ResultRelInfo);
@@ -137,8 +139,7 @@ static void begin_cypher_merge(CustomScanState *node, EState *estate,
         // Setup the relation's tuple slot
         cypher_node->elemTupleSlot = ExecInitExtraTupleSlot(
             estate,
-            RelationGetDescr(cypher_node->resultRelInfo->ri_RelationDesc),
-            &TTSOpsHeapTuple);
+            RelationGetDescr(cypher_node->resultRelInfo->ri_RelationDesc));
 
         if (cypher_node->id_expr != NULL)
         {
@@ -161,9 +162,7 @@ static void begin_cypher_merge(CustomScanState *node, EState *estate,
      * that have modified the command id.
      */
     if (estate->es_output_cid == 0)
-    {
         estate->es_output_cid = estate->es_snapshot->curcid;
-    }
 
     /* store the currentCommandId for this instance */
     css->base_currentCommandId = GetCurrentCommandId(false);
@@ -190,7 +189,7 @@ static bool check_path(cypher_merge_custom_scan_state *css,
          * If target_node as a valid attribute number and is a node not
          * declared in a previous clause, check the tuple position in the
          * slot. If the slot is null, the path was not found. The rules
-         * state that if one part of the path does not exists, the whold
+         * state that if one part of the path does not exists, the whole
          * path must be created.
          */
         if (node->tuple_position != InvalidAttrNumber ||
@@ -263,6 +262,7 @@ static void process_simple_merge(CustomScanState *node)
 
         /* setup the scantuple that the process_path needs */
         econtext->ecxt_scantuple = node->ss.ps.lefttree->ps_ResultTupleSlot;
+        econtext->ecxt_scantuple->tts_isempty = false;
 
         process_path(css);
     }
@@ -462,6 +462,7 @@ static TupleTableSlot *exec_cypher_merge(CustomScanState *node)
              */
             ExprContext *econtext = node->ss.ps.ps_ExprContext;
             SubqueryScanState *sss = (SubqueryScanState *)node->ss.ps.lefttree;
+            HeapTuple heap_tuple;
 
             /*
              * Our child execution node is always a subquery. If not there
@@ -476,7 +477,7 @@ static TupleTableSlot *exec_cypher_merge(CustomScanState *node)
             Assert(css->found_a_path == false);
 
             /*
-             * This block of sub-case 1 should only be exectuted once. To
+             * This block of sub-case 1 should only be executued once. To
              * create the single path if the path does not exist. If we find
              * ourselves here again, the internal state of the MERGE execution
              * node was incorrectly altered.
@@ -488,8 +489,8 @@ static TupleTableSlot *exec_cypher_merge(CustomScanState *node)
              *  it.
              */
             ExecInitScanTupleSlot(estate, &sss->ss,
-                                  ExecGetResultType(sss->subplan),
-                                  &TTSOpsVirtual);
+                                  ExecGetResultType(sss->subplan));
+
 
             /* setup the scantuple that the process_path needs */
             econtext->ecxt_scantuple = sss->ss.ss_ScanTupleSlot;
@@ -506,8 +507,14 @@ static TupleTableSlot *exec_cypher_merge(CustomScanState *node)
              */
             mark_tts_isnull(econtext->ecxt_scantuple);
 
-            // store the heap tuble
-            ExecStoreVirtualTuple(econtext->ecxt_scantuple);
+            // create the physical heap tuple
+            heap_tuple = heap_form_tuple(
+                                econtext->ecxt_scantuple->tts_tupleDescriptor,
+                                econtext->ecxt_scantuple->tts_values,
+                                econtext->ecxt_scantuple->tts_isnull);
+
+            // store the heap tuple
+            ExecStoreTuple(heap_tuple, econtext->ecxt_scantuple, InvalidBuffer, false);
 
             /*
              * make the subquery's projection scan slot be the tuple table we
@@ -557,8 +564,8 @@ static void end_cypher_merge(CustomScanState *node)
         ExecCloseIndices(cypher_node->resultRelInfo);
 
         // close the relation itself
-        table_close(cypher_node->resultRelInfo->ri_RelationDesc,
-                    RowExclusiveLock);
+        heap_close(cypher_node->resultRelInfo->ri_RelationDesc,
+                   RowExclusiveLock);
     }
 }
 
