@@ -19,20 +19,25 @@
 
 #include "postgres.h"
 
-#include "access/heapam.h"
+#include "access/sysattr.h"
 #include "access/htup_details.h"
 #include "access/xact.h"
+#include "storage/bufmgr.h"
 #include "executor/tuptable.h"
 #include "nodes/execnodes.h"
 #include "nodes/extensible.h"
 #include "nodes/nodes.h"
 #include "nodes/plannodes.h"
+#include "parser/parsetree.h"
+#include "parser/parse_relation.h"
 #include "rewrite/rewriteHandler.h"
-#include "storage/bufmgr.h"
 #include "utils/rel.h"
 
+#include "catalog/ag_label.h"
+#include "commands/label_commands.h"
 #include "executor/cypher_executor.h"
 #include "executor/cypher_utils.h"
+#include "parser/cypher_parse_node.h"
 #include "nodes/cypher_nodes.h"
 #include "utils/agtype.h"
 #include "utils/graphid.h"
@@ -77,8 +82,7 @@ static void begin_cypher_set(CustomScanState *node, EState *estate,
     ExecAssignExprContext(estate, &node->ss.ps);
 
     ExecInitScanTupleSlot(estate, &node->ss,
-                          ExecGetResultType(node->ss.ps.lefttree),
-                          &TTSOpsHeapTuple);
+                          ExecGetResultType(node->ss.ps.lefttree));
 
     if (!CYPHER_CLAUSE_IS_TERMINAL(css->flags))
     {
@@ -108,13 +112,12 @@ static HeapTuple update_entity_tuple(ResultRelInfo *resultRelInfo,
 {
     HeapTuple tuple = NULL;
     LockTupleMode lockmode;
-    TM_FailureData hufd;
-    TM_Result lock_result;
+    HeapUpdateFailureData hufd;
+    HTSU_Result lock_result;
+    HTSU_Result update_result;
     Buffer buffer;
-    bool update_indexes;
-    TM_Result   result;
 
-    ResultRelInfo *saved_resultRelInfo  = estate->es_result_relation_info;
+    ResultRelInfo *saved_resultRelInfo = estate->es_result_relation_info;
     estate->es_result_relation_info = resultRelInfo;
 
     lockmode = ExecUpdateLockMode(estate, resultRelInfo);
@@ -123,11 +126,10 @@ static HeapTuple update_entity_tuple(ResultRelInfo *resultRelInfo,
                                   GetCurrentCommandId(false), lockmode,
                                   LockWaitBlock, false, &buffer, &hufd);
 
-    if (lock_result == TM_Ok)
+    if (lock_result == HeapTupleMayBeUpdated)
     {
-        ExecOpenIndices(resultRelInfo, false);
         ExecStoreVirtualTuple(elemTupleSlot);
-        tuple = ExecFetchSlotHeapTuple(elemTupleSlot, true, NULL);
+        tuple = ExecMaterializeSlot(elemTupleSlot);
         tuple->t_self = old_tuple->t_self;
 
         // Check the constraints of the tuple
@@ -137,59 +139,26 @@ static HeapTuple update_entity_tuple(ResultRelInfo *resultRelInfo,
             ExecConstraints(resultRelInfo, elemTupleSlot, estate);
         }
 
-        result = table_tuple_update(resultRelInfo->ri_RelationDesc,
-                                    &tuple->t_self, elemTupleSlot,
+        // Insert the tuple normally
+        update_result = heap_update(resultRelInfo->ri_RelationDesc,
+                                    &(tuple->t_self), tuple,
                                     GetCurrentCommandId(true),
-                                    //estate->es_output_cid,
-                                    estate->es_snapshot,// NULL,
-                                    estate->es_crosscheck_snapshot,
-                                    true /* wait for commit */ ,
-                                    &hufd, &lockmode, &update_indexes);
+                                    estate->es_crosscheck_snapshot, true, &hufd,
+                                    &lockmode);
 
-        if (result == TM_SelfModified)
-        {
-            if (hufd.cmax != estate->es_output_cid)
-            {
-                ereport(ERROR,
-                        (errcode(ERRCODE_TRIGGERED_DATA_CHANGE_VIOLATION),
-                         errmsg("tuple to be updated was already modified")));
-            }
-
-            ExecCloseIndices(resultRelInfo);
-            estate->es_result_relation_info = saved_resultRelInfo;
-
-            return tuple;
-        }
-
-        if (result != TM_Ok)
+        if (update_result != HeapTupleMayBeUpdated)
         {
             ereport(ERROR, (errcode(ERRCODE_INTERNAL_ERROR),
-                    errmsg("Entity failed to be updated: %i", result)));
+                    errmsg("Entity failed to be updated: %i", update_result)));
         }
 
         // Insert index entries for the tuple
-        if (resultRelInfo->ri_NumIndices > 0 && update_indexes)
+        if (resultRelInfo->ri_NumIndices > 0)
         {
-          ExecInsertIndexTuples(elemTupleSlot, estate, false, NULL, NIL);
-        }
-
-            ExecCloseIndices(resultRelInfo);
-    }
-    else if (lock_result == TM_SelfModified)
-    {
-        if (hufd.cmax != estate->es_output_cid)
-        {
-            ereport(ERROR,
-                    (errcode(ERRCODE_TRIGGERED_DATA_CHANGE_VIOLATION),
-                     errmsg("tuple to be updated was already modified")));
+            ExecInsertIndexTuples(elemTupleSlot, &(tuple->t_self), estate,
+                                  false, NULL, NIL);
         }
     }
-    else
-    {
-        ereport(ERROR, (errcode(ERRCODE_INTERNAL_ERROR),
-                errmsg("Entity failed to be updated: %i", lock_result)));
-    }
-
     ReleaseBuffer(buffer);
 
     estate->es_result_relation_info = saved_resultRelInfo;
@@ -242,7 +211,7 @@ static bool check_path(agtype_value *path, graphid updated_id)
 
 /*
  * Construct a new agtype path with the entity with updated_id
- * replacing all of its intances in path with updated_entity
+ * replacing all of its instances in path with updated_entity
  */
 static agtype_value *replace_entity_in_path(agtype_value *path,
                                             graphid updated_id,
@@ -407,14 +376,13 @@ static void process_update_list(CustomScanState *node)
         TupleTableSlot *slot;
         ResultRelInfo *resultRelInfo;
         ScanKeyData scan_keys[1];
-        TableScanDesc scan_desc;
+        HeapScanDesc scan_desc;
         bool remove_property;
         char *label_name;
         cypher_update_item *update_item;
         Datum new_entity;
         HeapTuple heap_tuple;
         char *clause_name = css->set_list->clause_name;
-        int cid;
 
         update_item = (cypher_update_item *)lfirst(lc);
 
@@ -450,8 +418,8 @@ static void process_update_list(CustomScanState *node)
         /* get the id and label for later */
         id = GET_AGTYPE_VALUE_OBJECT_VALUE(original_entity_value, "id");
         label = GET_AGTYPE_VALUE_OBJECT_VALUE(original_entity_value, "label");
-
         label_name = pnstrdup(label->val.string.val, label->val.string.len);
+
         /* get the properties we need to update */
         original_properties = GET_AGTYPE_VALUE_OBJECT_VALUE(original_entity_value,
                                                             "properties");
@@ -473,7 +441,7 @@ static void process_update_list(CustomScanState *node)
 
         /*
          * If we need to remove the property, set the value to NULL. Otherwise
-         * fetch the evaluated expression from the tuble slot.
+         * fetch the evaluated expression from the tuple slot.
          */
         if (remove_property)
         {
@@ -484,21 +452,24 @@ static void process_update_list(CustomScanState *node)
             new_property_value = DATUM_GET_AGTYPE_P(scanTupleSlot->tts_values[update_item->prop_position - 1]);
         }
 
-        /*
-         * Alter the properties Agtype value to contain or remove the updated
-         * property.
-         */
-        altered_properties = alter_property_value(original_properties,
-                                                  update_item->prop_name,
-                                                  new_property_value,
-                                                  remove_property);
+        // Alter the properties Agtype value.
+        if (strcmp(update_item->prop_name, ""))
+        {
+            altered_properties = alter_property_value(original_properties,
+                                                      update_item->prop_name,
+                                                      new_property_value,
+                                                      remove_property);
+        }
+        else
+        {
+            altered_properties = get_map_from_agtype(new_property_value);
+        }
 
-        resultRelInfo = create_entity_result_rel_info(
-            estate, css->set_list->graph_name, label_name);
+        resultRelInfo = create_entity_result_rel_info(estate,
+                                                      css->set_list->graph_name,
+                                                      label_name);
 
-        slot = ExecInitExtraTupleSlot(
-            estate, RelationGetDescr(resultRelInfo->ri_RelationDesc),
-            &TTSOpsHeapTuple);
+        slot = ExecInitExtraTupleSlot(estate, RelationGetDescr(resultRelInfo->ri_RelationDesc));
 
         /*
          *  Now that we have the updated properties, create a either a vertex or
@@ -549,9 +520,6 @@ static void process_update_list(CustomScanState *node)
          * If the last update index for the entity is equal to the current loop
          * index, then update this tuple.
          */
-        cid = estate->es_snapshot->curcid;
-        estate->es_snapshot->curcid = GetCurrentCommandId(false);
-
         if (luindex[update_item->entity_position - 1] == lidx)
         {
             /*
@@ -564,8 +532,8 @@ static void process_update_list(CustomScanState *node)
              * Setup the scan description, with the correct snapshot and scan
              * keys.
              */
-            scan_desc = table_beginscan(resultRelInfo->ri_RelationDesc,
-                                        estate->es_snapshot, 1, scan_keys);
+            scan_desc = heap_beginscan(resultRelInfo->ri_RelationDesc,
+                                       estate->es_snapshot, 1, scan_keys);
             /* Retrieve the tuple. */
             heap_tuple = heap_getnext(scan_desc, ForwardScanDirection);
 
@@ -579,13 +547,12 @@ static void process_update_list(CustomScanState *node)
                                                  heap_tuple);
             }
             /* close the ScanDescription */
-            table_endscan(scan_desc);
+            heap_endscan(scan_desc);
         }
 
-        estate->es_snapshot->curcid = cid;
         /* close relation */
         ExecCloseIndices(resultRelInfo);
-        table_close(resultRelInfo->ri_RelationDesc, RowExclusiveLock);
+        heap_close(resultRelInfo->ri_RelationDesc, RowExclusiveLock);
 
         /* increment loop index */
         lidx++;
