@@ -296,6 +296,17 @@ void out_cypher_sub_pattern(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(pattern);
 }
 
+// serialization function for the cypher_call ExtensibleNode.
+void out_cypher_call(StringInfo str, const ExtensibleNode *node)
+{
+    DEFINE_AG_NODE(cypher_call);
+
+    WRITE_NODE_FIELD(funccall);
+    WRITE_NODE_FIELD(funcexpr);
+    WRITE_NODE_FIELD(where);
+    WRITE_NODE_FIELD(yield_items);
+}
+
 // serialization function for the cypher_create_target_nodes ExtensibleNode.
 void out_cypher_create_target_nodes(StringInfo str, const ExtensibleNode *node)
 {
@@ -303,7 +314,7 @@ void out_cypher_create_target_nodes(StringInfo str, const ExtensibleNode *node)
 
     WRITE_NODE_FIELD(paths);
     WRITE_INT32_FIELD(flags);
-    WRITE_INT32_FIELD(graph_oid);
+    WRITE_OID_FIELD(graph_oid);
 }
 
 // serialization function for the cypher_create_path ExtensibleNode.
@@ -370,7 +381,7 @@ void out_cypher_delete_information(StringInfo str, const ExtensibleNode *node)
     WRITE_NODE_FIELD(delete_items);
     WRITE_INT32_FIELD(flags);
     WRITE_STRING_FIELD(graph_name);
-    WRITE_INT32_FIELD(graph_oid);
+    WRITE_OID_FIELD(graph_oid);
     WRITE_BOOL_FIELD(detach);
 }
 
@@ -389,7 +400,7 @@ void out_cypher_merge_information(StringInfo str, const ExtensibleNode *node)
     DEFINE_AG_NODE(cypher_merge_information);
 
     WRITE_INT32_FIELD(flags);
-    WRITE_INT32_FIELD(graph_oid);
+    WRITE_OID_FIELD(graph_oid);
     WRITE_INT32_FIELD(merge_function_attr);
     WRITE_NODE_FIELD(path);
 }
