@@ -32,6 +32,7 @@
 #include "parser/parse_node.h"
 #include "parser/parse_relation.h"
 #include "parser/parse_target.h"
+#include "parser/parsetree.h"
 #include "utils/builtins.h"
 
 #include "catalog/ag_graph.h"
@@ -44,7 +45,17 @@
 #include "utils/age_session_info.h"
 #include "utils/agtype.h"
 
+/*
+ * extra_node is a global variable to this source to store, at the moment, the
+ * explain stmt node passed up by the parser. The return value from the parser
+ * contains an 'extra' value, hence the name.
+ */
 static Node *extra_node = NULL;
+/*
+ * Takes a query node and builds an explain stmt query node. It then replaces
+ * the passed query node with the new explain stmt query node.
+ */
+static void build_explain_query(Query *query, Node *explain_node);
 
 static post_parse_analyze_hook_type prev_post_parse_analyze_hook;
 
@@ -58,13 +69,14 @@ static const char *expr_get_const_cstring(Node *expr, const char *source_str);
 static int get_query_location(const int location, const char *source_str);
 static Query *analyze_cypher(List *stmt, ParseState *parent_pstate,
                              const char *query_str, int query_loc,
-                             char *graph_name, uint32 graph_oid, Param *params);
+                             char *graph_name, Oid graph_oid, Param *params);
 static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
                                         ParseState *parent_pstate,
                                         const char *query_str, int query_loc,
-                                        char *graph_name, uint32 graph_oid,
+                                        char *graph_name, Oid graph_oid,
                                         Param *params);
 
+
 void post_parse_analyze_init(void)
 {
     prev_post_parse_analyze_hook = post_parse_analyze_hook;
@@ -79,9 +91,34 @@ void post_parse_analyze_fini(void)
 static void post_parse_analyze(ParseState *pstate, Query *query)
 {
     if (prev_post_parse_analyze_hook)
+    {
         prev_post_parse_analyze_hook(pstate, query);
+    }
+
+    /*
+     * extra_node is set in the parsing stage to keep track of EXPLAIN.
+     * So it needs to be set to NULL prior to any cypher parsing.
+     */
+    extra_node = NULL;
 
     convert_cypher_walker((Node *)query, pstate);
+
+    /*
+     * If there is an extra_node returned, we need to check to see if
+     * it is an EXPLAIN.
+     */
+    if (extra_node != NULL)
+    {
+        /* process the EXPLAIN node */
+        if (nodeTag(extra_node) == T_ExplainStmt)
+        {
+            build_explain_query(query, extra_node);
+        }
+
+        /* reset extra_node */
+        pfree(extra_node);
+        extra_node = NULL;
+    }
 }
 
 // find cypher() calls in FROM clauses and convert them to SELECT subqueries
@@ -175,62 +212,60 @@ static bool convert_cypher_walker(Node *node, ParseState *pstate)
          * QTW_IGNORE_JOINALIASES
          *     We are not interested in this.
          */
-        flags = QTW_EXAMINE_RTES_BEFORE | QTW_IGNORE_RT_SUBQUERIES |
+        flags = QTW_EXAMINE_RTES | QTW_IGNORE_RT_SUBQUERIES |
                 QTW_IGNORE_JOINALIASES;
 
-        /* clear the global variable extra_node */
-        extra_node = NULL;
-
         /* recurse on query */
         result = query_tree_walker(query, convert_cypher_walker, pstate, flags);
 
-        /* check for EXPLAIN */
-        if (extra_node != NULL && nodeTag(extra_node) == T_ExplainStmt)
-        {
-            ExplainStmt *estmt = NULL;
-            Query *query_copy = NULL;
-            Query *query_node = NULL;
-
-            /*
-             * Create a copy of the query node. This is purposely a shallow copy
-             * because we are only moving the contents to another pointer.
-             */
-            query_copy = (Query *) palloc(sizeof(Query));
-            memcpy(query_copy, query, sizeof(Query));
-
-            /* build our Explain node and store the query node copy in it */
-            estmt = makeNode(ExplainStmt);
-            estmt->query = (Node *)query_copy;
-            estmt->options = ((ExplainStmt *)extra_node)->options;
-
-            /* build our replacement query node */
-            query_node = makeNode(Query);
-            query_node->commandType = CMD_UTILITY;
-            query_node->utilityStmt = (Node *)estmt;
-            query_node->canSetTag = true;
-
-            /* now replace the top query node with our replacement query node */
-            memcpy(query, query_node, sizeof(Query));
-
-            /*
-             * We need to free and clear the global variable when done. But, not
-             * the ExplainStmt options. Those will get freed by PG when the
-             * query is deleted.
-             */
-            ((ExplainStmt *)extra_node)->options = NULL;
-            pfree(extra_node);
-            extra_node = NULL;
-
-            /* we need to free query_node as it is no longer needed */
-            pfree(query_node);
-        }
-
         return result;
     }
 
     return expression_tree_walker(node, convert_cypher_walker, pstate);
 }
 
+/*
+ * Takes a query node and builds an explain stmt query node. It then replaces
+ * the passed query node with the new explain stmt query node.
+ */
+static void build_explain_query(Query *query, Node *explain_node)
+{
+    ExplainStmt *estmt = NULL;
+    Query *query_copy = NULL;
+    Query *query_node = NULL;
+
+    /*
+     * Create a copy of the query node. This is purposely a shallow copy
+     * because we are only moving the contents to another pointer.
+     */
+    query_copy = (Query *) palloc(sizeof(Query));
+    memcpy(query_copy, query, sizeof(Query));
+
+    /* build our Explain node and store the query node copy in it */
+    estmt = makeNode(ExplainStmt);
+    estmt->query = (Node *)query_copy;
+    estmt->options = ((ExplainStmt *)explain_node)->options;
+
+    /* build our replacement query node */
+    query_node = makeNode(Query);
+    query_node->commandType = CMD_UTILITY;
+    query_node->utilityStmt = (Node *)estmt;
+    query_node->canSetTag = true;
+
+    /* now replace the top query node with our replacement query node */
+    memcpy(query, query_node, sizeof(Query));
+
+    /*
+     * We need to free and clear the global variable when done. But, not
+     * the ExplainStmt options. Those will get freed by PG when the
+     * query is deleted.
+     */
+    ((ExplainStmt *)explain_node)->options = NULL;
+
+    /* we need to free query_node as it is no longer needed */
+    pfree(query_node);
+}
+
 static bool is_rte_cypher(RangeTblEntry *rte)
 {
     RangeTblFunction *rtfunc;
@@ -274,7 +309,7 @@ static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
     Node *arg3 = NULL;
     Name graph_name = NULL;
     char *graph_name_str = NULL;
-    uint32 graph_oid;
+    Oid graph_oid = InvalidOid;
     const char *query_str = NULL;
     int query_loc = -1;
     Param *params = NULL;
@@ -317,7 +352,6 @@ static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
      *   may differ from what they are shown. This will confuse users.
      * * In the case above, the error position may not be accurate.
      */
-
     query_str = expr_get_const_cstring(arg2, pstate->p_sourcetext);
 
     /*
@@ -428,7 +462,7 @@ static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
     }
 
     /*
-     * Check to see if the cypher function had a third parameters passed to it,
+     * Check to see if the cypher function had a third parameter passed to it,
      * if so make sure Postgres parsed the second argument to a Param node.
      */
     if (list_length(funcexpr->args) == 3)
@@ -475,6 +509,10 @@ static void convert_cypher_to_subquery(RangeTblEntry *rte, ParseState *pstate)
     {
         Node *temp = llast(stmt);
 
+        ereport(WARNING,
+                (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+                 errmsg("too many extra_nodes passed from parser")));
+
         list_delete_ptr(stmt, temp);
     }
 
@@ -574,7 +612,7 @@ static int get_query_location(const int location, const char *source_str)
 
 static Query *analyze_cypher(List *stmt, ParseState *parent_pstate,
                              const char *query_str, int query_loc,
-                             char *graph_name, uint32 graph_oid, Param *params)
+                             char *graph_name, Oid graph_oid, Param *params)
 {
     cypher_clause *clause;
     ListCell *lc;
@@ -653,7 +691,7 @@ static Query *analyze_cypher(List *stmt, ParseState *parent_pstate,
 static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
                                         ParseState *parent_pstate,
                                         const char *query_str, int query_loc,
-                                        char *graph_name, uint32 graph_oid,
+                                        char *graph_name, Oid graph_oid,
                                         Param *params)
 {
     ParseState *pstate;
@@ -739,7 +777,7 @@ static Query *analyze_cypher_and_coerce(List *stmt, RangeTblFunction *rtfunc,
 
             /*
              * The coercion context of this coercion is COERCION_EXPLICIT
-             * because the target type is explicitly metioned in the column
+             * because the target type is explicitly mentioned in the column
              * definition list and we need to do this by looking up all
              * possible coercion.
              */
