@@ -87,6 +87,18 @@ typedef struct label_relation_cache_entry
     label_cache_data data;
 } label_relation_cache_entry;
 
+typedef struct label_seq_name_graph_cache_key
+{
+    NameData name;
+    Oid graph;
+} label_seq_name_graph_cache_key;
+
+typedef struct label_seq_name_graph_cache_entry
+{
+    label_seq_name_graph_cache_key key; // hash key
+    label_cache_data data;
+} label_seq_name_graph_cache_entry;
+
 // ag_graph.name
 static HTAB *graph_name_cache_hash = NULL;
 static ScanKeyData graph_name_scan_keys[1];
@@ -111,6 +123,10 @@ static ScanKeyData label_graph_id_scan_keys[2];
 static HTAB *label_relation_cache_hash = NULL;
 static ScanKeyData label_relation_scan_keys[1];
 
+// ag_label.seq_name, ag_label.graph
+static HTAB *label_seq_name_graph_cache_hash = NULL;
+static ScanKeyData label_seq_name_graph_scan_keys[2];
+
 // initialize all caches
 static void initialize_caches(void);
 
@@ -140,6 +156,7 @@ static void create_label_oid_cache(void);
 static void create_label_name_graph_cache(void);
 static void create_label_graph_id_cache(void);
 static void create_label_relation_cache(void);
+static void create_label_seq_name_graph_cache(void);
 static void invalidate_label_caches(Datum arg, Oid relid);
 static void invalidate_label_oid_cache(Oid relid);
 static void flush_label_oid_cache(void);
@@ -149,6 +166,9 @@ static void invalidate_label_graph_id_cache(Oid relid);
 static void flush_label_graph_id_cache(void);
 static void invalidate_label_relation_cache(Oid relid);
 static void flush_label_relation_cache(void);
+static void invalidate_label_seq_name_graph_cache(Oid relid);
+static void flush_label_seq_name_graph_cache(void);
+
 static label_cache_data *search_label_oid_cache_miss(Oid oid);
 static label_cache_data *search_label_name_graph_cache_miss(Name name,
                                                             Oid graph);
@@ -159,6 +179,12 @@ static label_cache_data *search_label_graph_id_cache_miss(Oid graph, int32 id);
 static void *label_graph_id_cache_hash_search(Oid graph, int32 id,
                                               HASHACTION action, bool *found);
 static label_cache_data *search_label_relation_cache_miss(Oid relation);
+static label_cache_data *search_label_seq_name_graph_cache_miss(Name name,
+                                                            Oid graph);
+static void *label_seq_name_graph_cache_hash_search(Name name, Oid graph,
+                                                HASHACTION action,
+                                                bool *found);
+
 static void fill_label_cache_data(label_cache_data *cache_data,
                                   HeapTuple tuple, TupleDesc tuple_desc);
 
@@ -486,6 +512,12 @@ static void initialize_label_caches(void)
     // ag_label.relation
     ag_cache_scan_key_init(&label_relation_scan_keys[0],
                            Anum_ag_label_relation, F_OIDEQ);
+    
+    // ag_label.seq_name, ag_label.graph
+    ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[0], Anum_ag_label_seq_name,
+                           F_NAMEEQ);
+    ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[1], Anum_ag_label_graph,
+                           F_OIDEQ);
 
     create_label_caches();
 
@@ -506,6 +538,8 @@ static void create_label_caches(void)
     create_label_name_graph_cache();
     create_label_graph_id_cache();
     create_label_relation_cache();
+    create_label_seq_name_graph_cache();
+
 }
 
 static void create_label_oid_cache(void)
@@ -578,9 +612,28 @@ static void create_label_relation_cache(void)
                                             &hash_ctl, HASH_ELEM | HASH_BLOBS);
 }
 
+static void create_label_seq_name_graph_cache(void)
+{
+    HASHCTL hash_ctl;
+
+    MemSet(&hash_ctl, 0, sizeof(hash_ctl));
+    hash_ctl.keysize = sizeof(label_seq_name_graph_cache_key);
+    hash_ctl.entrysize = sizeof(label_seq_name_graph_cache_entry);
+
+    /*
+     * Please see the comment of hash_create() for the nelem value 16 here.
+     * HASH_BLOBS flag is set because the key for this hash is fixed-size.
+     */
+    label_seq_name_graph_cache_hash = hash_create("ag_label (seq_name, graph) cache",
+                                              16, &hash_ctl,
+                                              HASH_ELEM | HASH_BLOBS);
+}
+
 static void invalidate_label_caches(Datum arg, Oid relid)
 {
     Assert(label_name_graph_cache_hash);
+    Assert(label_seq_name_graph_cache_hash);
+
 
     if (OidIsValid(relid))
     {
@@ -588,6 +641,7 @@ static void invalidate_label_caches(Datum arg, Oid relid)
         invalidate_label_name_graph_cache(relid);
         invalidate_label_graph_id_cache(relid);
         invalidate_label_relation_cache(relid);
+        invalidate_label_seq_name_graph_cache(relid);
     }
     else
     {
@@ -595,6 +649,7 @@ static void invalidate_label_caches(Datum arg, Oid relid)
         flush_label_name_graph_cache();
         flush_label_graph_id_cache();
         flush_label_relation_cache();
+        flush_label_seq_name_graph_cache();
     }
 }
 
@@ -633,7 +688,7 @@ static void flush_label_oid_cache(void)
 {
     HASH_SEQ_STATUS hash_seq;
 
-    hash_seq_init(&hash_seq, label_name_graph_cache_hash);
+    hash_seq_init(&hash_seq, label_oid_cache_hash);
     for (;;)
     {
         label_cache_data *entry;
@@ -802,6 +857,61 @@ static void flush_label_relation_cache(void)
     }
 }
 
+static void invalidate_label_seq_name_graph_cache(Oid relid)
+{
+    HASH_SEQ_STATUS hash_seq;
+
+    hash_seq_init(&hash_seq, label_seq_name_graph_cache_hash);
+    for (;;)
+    {
+        label_seq_name_graph_cache_entry *entry;
+        void *removed;
+
+        entry = hash_seq_search(&hash_seq);
+        if (!entry)
+            break;
+
+        if (entry->data.relation != relid)
+            continue;
+
+        removed = hash_search(label_seq_name_graph_cache_hash, &entry->key,
+                              HASH_REMOVE, NULL);
+        hash_seq_term(&hash_seq);
+
+        if (!removed)
+        {
+            ereport(ERROR,
+                    (errmsg_internal("label (seq_name, graph) cache corrupted")));
+        }
+
+        break;
+    }
+}
+
+static void flush_label_seq_name_graph_cache(void)
+{
+    HASH_SEQ_STATUS hash_seq;
+
+    hash_seq_init(&hash_seq, label_seq_name_graph_cache_hash);
+    for (;;)
+    {
+        label_seq_name_graph_cache_entry *entry;
+        void *removed;
+
+        entry = hash_seq_search(&hash_seq);
+        if (!entry)
+            break;
+
+        removed = hash_search(label_seq_name_graph_cache_hash, &entry->key,
+                              HASH_REMOVE, NULL);
+        if (!removed)
+        {
+            ereport(ERROR,
+                    (errmsg_internal("label (seq_name, graph) cache corrupted")));
+        }
+    }
+}
+
 label_cache_data *search_label_oid_cache(Oid oid)
 {
     label_cache_data *entry;
@@ -1080,6 +1190,88 @@ static label_cache_data *search_label_relation_cache_miss(Oid relation)
     return entry;
 }
 
+label_cache_data *search_label_seq_name_graph_cache(const char *name, Oid graph)
+{
+    NameData name_key;
+    label_seq_name_graph_cache_entry *entry;
+
+    AssertArg(name);
+    AssertArg(OidIsValid(graph));
+
+    initialize_caches();
+
+    namestrcpy(&name_key, name);
+    entry = label_seq_name_graph_cache_hash_search(&name_key, graph, HASH_FIND,
+                                               NULL);
+    if (entry)
+        return &entry->data;
+
+    return search_label_seq_name_graph_cache_miss(&name_key, graph);
+}
+
+static label_cache_data *search_label_seq_name_graph_cache_miss(Name name,
+                                                            Oid graph)
+{
+    ScanKeyData scan_keys[2];
+    Relation ag_label;
+    SysScanDesc scan_desc;
+    HeapTuple tuple;
+    bool found;
+    label_seq_name_graph_cache_entry *entry;
+
+    memcpy(scan_keys, label_seq_name_graph_scan_keys,
+           sizeof(label_seq_name_graph_scan_keys));
+    scan_keys[0].sk_argument = NameGetDatum(name);
+    scan_keys[1].sk_argument = ObjectIdGetDatum(graph);
+
+    /*
+     * Calling heap_open() might call AcceptInvalidationMessage() and that
+     * might invalidate the label caches. This is OK because this function is
+     * called when the desired entry is not in the cache.
+     */
+    ag_label = heap_open(ag_label_relation_id(), AccessShareLock);
+    scan_desc = systable_beginscan(ag_label, ag_label_seq_name_graph_index_id(),
+                                   true, NULL, 2, scan_keys);
+
+    /*
+     * don't need to loop over scan_desc because ag_label_seq_name_graph_index is
+     * UNIQUE
+     */
+    tuple = systable_getnext(scan_desc);
+    if (!HeapTupleIsValid(tuple))
+    {
+        systable_endscan(scan_desc);
+        heap_close(ag_label, AccessShareLock);
+
+        return NULL;
+    }
+
+    // get a new entry
+    entry = label_seq_name_graph_cache_hash_search(name, graph, HASH_ENTER,
+                                               &found);
+    Assert(!found); // no concurrent update on label_seq_name_graph_cache_hash
+
+    // fill the new entry with the retrieved tuple
+    fill_label_cache_data(&entry->data, tuple, RelationGetDescr(ag_label));
+
+    systable_endscan(scan_desc);
+    heap_close(ag_label, AccessShareLock);
+
+    return &entry->data;
+}
+
+static void *label_seq_name_graph_cache_hash_search(Name name, Oid graph,
+                                                HASHACTION action, bool *found)
+{
+    label_seq_name_graph_cache_key key;
+
+    // initialize the hash key for label_seq_name_graph_cache_hash
+    namecpy(&key.name, name);
+    key.graph = graph;
+
+    return hash_search(label_seq_name_graph_cache_hash, &key, action, found);
+}
+
 static void fill_label_cache_data(label_cache_data *cache_data,
                                   HeapTuple tuple, TupleDesc tuple_desc)
 {
@@ -1110,4 +1302,8 @@ static void fill_label_cache_data(label_cache_data *cache_data,
     value = heap_getattr(tuple, Anum_ag_label_relation, tuple_desc, &is_null);
     Assert(!is_null);
     cache_data->relation = DatumGetObjectId(value);
+    // ag_label.seq_name
+    value = heap_getattr(tuple, Anum_ag_label_seq_name, tuple_desc, &is_null);
+    Assert(!is_null);
+    namecpy(&cache_data->seq_name, DatumGetName(value));
 }
