@@ -49,37 +49,42 @@
 #define Anum_ag_label_id 3
 #define Anum_ag_label_kind 4
 #define Anum_ag_label_relation 5
+#define Anum_ag_label_seq_name 6
 
-#define Natts_ag_label 5
+
+#define Natts_ag_label 6
 
 #define ag_label_relation_id() ag_relation_id("ag_label", "table")
+#define ag_label_oid_index_id() ag_relation_id("ag_label_oid_index", "index")
 #define ag_label_name_graph_index_id() \
     ag_relation_id("ag_label_name_graph_index", "index")
-#define ag_label_graph_oid_index_id() \
-    ag_relation_id("ag_label_graph_oid_index", "index")
+#define ag_label_graph_id_index_id() \
+    ag_relation_id("ag_label_graph_id_index", "index")
 #define ag_label_relation_index_id() \
     ag_relation_id("ag_label_relation_index", "index")
+#define ag_label_seq_name_graph_index_id() \
+    ag_relation_id("ag_label_seq_name_graph_index", "index")
 
 #define LABEL_ID_SEQ_NAME "_label_id_seq"
 
 #define LABEL_KIND_VERTEX 'v'
 #define LABEL_KIND_EDGE 'e'
 
-void insert_label(const char *label_name, Oid graph_oid, int32 label_id,
-                  char label_kind, Oid label_relation);
+Oid insert_label(const char *label_name, Oid label_graph, int32 label_id,
+                 char label_kind, Oid label_relation, const char *seq_name);
 void delete_label(Oid relation);
 
-int32 get_label_id(const char *label_name, Oid graph_oid);
-Oid get_label_relation(const char *label_name, Oid graph_oid);
-char *get_label_relation_name(const char *label_name, Oid graph_oid);
+Oid get_label_oid(const char *label_name, Oid label_graph);
+int32 get_label_id(const char *label_name, Oid label_graph);
+Oid get_label_relation(const char *label_name, Oid label_graph);
+char *get_label_relation_name(const char *label_name, Oid label_graph);
 
-bool label_id_exists(Oid graph_oid, int32 label_id);
-RangeVar *get_label_range_var(char *graph_name, Oid graph_oid,
-                              char *label_name);
+bool label_id_exists(Oid label_graph, int32 label_id);
+RangeVar *get_label_range_var(char *graph_name, Oid graph_oid, char *label_name);
 
 List *get_all_edge_labels_per_graph(EState *estate, Oid graph_oid);
 
 #define label_exists(label_name, label_graph) \
-    OidIsValid(get_label_id(label_name, label_graph))
+    OidIsValid(get_label_oid(label_name, label_graph))
 
 #endif
