@@ -225,7 +225,7 @@ typedef struct cypher_create_target_nodes
     ExtensibleNode extensible;
     List *paths;
     uint32 flags;
-    uint32 graph_oid;
+    Oid graph_oid;
 } cypher_create_target_nodes;
 
 typedef struct cypher_create_path
@@ -236,6 +236,20 @@ typedef struct cypher_create_path
     char *var_name;
 } cypher_create_path;
 
+/*
+ * procedure call
+ */
+
+typedef struct cypher_call
+{
+    ExtensibleNode extensible;
+    FuncCall *funccall; /*from the parser */
+    FuncExpr *funcexpr; /*transformed */
+
+    Node *where;
+    List *yield_items; // optional yield subclause
+} cypher_call;
+
 #define CYPHER_CLAUSE_FLAG_NONE 0x0000
 #define CYPHER_CLAUSE_FLAG_TERMINAL 0x0001
 #define CYPHER_CLAUSE_FLAG_PREVIOUS_CLAUSE 0x0002
@@ -296,7 +310,7 @@ typedef struct cypher_target_node
     /*
      * Attribute number this entity needs to be stored in
      * for parent execution nodes to reference it. If the
-     * entity is a varaible (CYPHER_TARGET_NODE_IS_VAR).
+     * entity is a variable (CYPHER_TARGET_NODE_IS_VAR).
      */
     AttrNumber tuple_position;
 } cypher_target_node;
@@ -309,7 +323,7 @@ typedef struct cypher_target_node
  * a variable that was already created AND created in the
  * same clause.
  */
-#define EXISTING_VARAIBLE_DECLARED_SAME_CLAUSE 0x0002
+#define EXISTING_VARIABLE_DECLARED_SAME_CLAUSE 0x0002
 
 //node is the first instance of a declared variable
 #define CYPHER_TARGET_NODE_IS_VAR 0x0004
@@ -332,7 +346,7 @@ typedef struct cypher_target_node
  * later. We don't need to check to see if the vertex still exists.
  */
 #define SAFE_TO_SKIP_EXISTENCE_CHECK(flags) \
-    (flags & EXISTING_VARAIBLE_DECLARED_SAME_CLAUSE)
+    (flags & EXISTING_VARIABLE_DECLARED_SAME_CLAUSE)
 
 #define CYPHER_TARGET_NODE_INSERT_ENTITY(flags) \
     (flags & CYPHER_TARGET_NODE_FLAG_INSERT)
@@ -368,7 +382,7 @@ typedef struct cypher_delete_information
     List *delete_items;
     int flags;
     char *graph_name;
-    uint32 graph_oid;
+    Oid graph_oid;
     bool detach;
 } cypher_delete_information;
 
@@ -383,7 +397,7 @@ typedef struct cypher_merge_information
 {
     ExtensibleNode extensible;
     int flags;
-    uint32 graph_oid;
+    Oid graph_oid;
     AttrNumber merge_function_attr;
     cypher_create_path *path;
 } cypher_merge_information;
