@@ -31,8 +31,8 @@
 #ifndef AG_AGTYPE_H
 #define AG_AGTYPE_H
 
-#include "access/htup_details.h"
 #include "fmgr.h"
+#include "access/htup_details.h"
 #include "lib/stringinfo.h"
 #include "nodes/pg_list.h"
 #include "utils/array.h"
@@ -40,7 +40,6 @@
 #include "utils/syscache.h"
 
 #include "catalog/ag_namespace.h"
-#include "catalog/pg_type.h"
 #include "utils/graphid.h"
 
 /* Tokens used when sequentially processing an agtype value */
@@ -304,7 +303,7 @@ typedef struct
 
 /*
  * IMPORTANT NOTE: For agtype_value_type, IS_A_AGTYPE_SCALAR() checks that the
- * type is between AGTV_NULL and AGTV_BOOL, inclusive. So, new scalars need to
+ * type is between AGTV_NULL and AGTV_ARRAY, excluding AGTV_ARRAY. So, new scalars need to
  * be between these values.
  */
 enum agtype_value_type
@@ -524,6 +523,7 @@ bool is_decimal_needed(char *numstr);
 int compare_agtype_scalar_values(agtype_value *a, agtype_value *b);
 agtype_value *alter_property_value(agtype_value *properties, char *var_name,
                                    agtype *new_v, bool remove_property);
+agtype_value *get_map_from_agtype(agtype *a);
 
 agtype *get_one_agtype_from_variadic_args(FunctionCallInfo fcinfo,
                                           int variadic_offset,
